#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_template.template import template_config, ViewTemplate, get_view_template

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget, IHTMLParagraph, \
    IParagraphContainer, IBaseParagraph, IParagraphSummary
from pyams_content.shared.common.interfaces import IWfSharedContent
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu
from pyams_skin.layer import IPyAMSLayer

# import packages
from pyams_content.component.paragraph.html import HTMLParagraph
from pyams_content.component.paragraph.zmi.container import ParagraphContainerView
from pyams_form.form import AJAXAddForm, AJAXEditForm
from pyams_form.security import ProtectedFormObjectMixin
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.viewlet.toolbar import ToolbarMenuItem
from pyams_utils.adapter import adapter_config, ContextRequestAdapter
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config, ContentProvider
from pyams_zmi.form import AdminDialogAddForm, AdminDialogEditForm
from pyramid.view import view_config
from z3c.form import field

from pyams_content import _


#
# HTML paragraph
#

@viewlet_config(name='add-html-paragraph.menu', context=IParagraphContainerTarget, view=ParagraphContainerView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=50)
class HTMLParagraphAddMenu(ProtectedFormObjectMixin, ToolbarMenuItem):
    """HTML paragraph add menu"""

    label = _("Add HTML paragraph...")
    label_css_class = 'fa fa-fw fa-html5'
    url = 'add-html-paragraph.html'
    modal_target = True


@pagelet_config(name='add-html-paragraph.html', context=IParagraphContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class HTMLParagraphAddForm(AdminDialogAddForm):
    """HTML paragraph add form"""

    legend = _("Add new HTML paragraph")
    dialog_class = 'modal-max'
    icon_css_class = 'fa fa-fw fa-html5'
    label_css_class = 'control-label col-md-2'
    input_css_class = 'col-md-10'

    fields = field.Fields(IHTMLParagraph).omit('__parent__', '__name__')
    ajax_handler = 'add-html-paragraph.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(HTMLParagraphAddForm, self).updateWidgets(prefix)
        self.widgets['body'].label_css_class = 'textarea'

    def create(self, data):
        return HTMLParagraph()

    def add(self, object):
        IParagraphContainer(self.context)['none'] = object


@view_config(name='add-html-paragraph.json', context=IParagraphContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class HTMLParagraphAJAXAddForm(AJAXAddForm, HTMLParagraphAddForm):
    """HTML paragraph add form, JSON renderer"""

    def get_ajax_output(self, changes):
        return {'status': 'reload',
                'location': '#paragraphs.html'}


@pagelet_config(name='properties.html', context=IHTMLParagraph, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class HTMLParagraphPropertiesEditForm(AdminDialogEditForm):
    """HTML paragraph properties edit form"""

    @property
    def title(self):
        content = get_parent(self.context, IWfSharedContent)
        return II18n(content).query_attribute('title', request=self.request)

    legend = _("Edit paragraph properties")
    dialog_class = 'modal-max'
    icon_css_class = 'fa fa-fw fa-html5'
    label_css_class = 'control-label col-md-2'
    input_css_class = 'col-md-10'

    fields = field.Fields(IHTMLParagraph).omit('__parent__', '__name__')
    ajax_handler = 'properties.json'
    edit_permission = 'pyams.ManageContent'

    def updateWidgets(self, prefix=None):
        super(HTMLParagraphPropertiesEditForm, self).updateWidgets(prefix)
        self.widgets['body'].label_css_class = 'textarea'


@view_config(name='properties.json', context=IHTMLParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class HTMLParagraphPropertiesAJAXEditForm(AJAXEditForm, HTMLParagraphPropertiesEditForm):
    """HTML paragraph properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        if 'title' in changes.get(IBaseParagraph, ()):
            return {'status': 'reload',
                    'location': '#paragraphs.html'}
        else:
            return super(HTMLParagraphPropertiesAJAXEditForm, self).get_ajax_output(changes)


#
# HTML paragraph summary
#

@adapter_config(context=(IHTMLParagraph, IPyAMSLayer), provides=IParagraphSummary)
@template_config(template='templates/html-summary.pt', layer=IPyAMSLayer)
class HTMLParagraphSummary(ContextRequestAdapter):
    """HTML paragraph renderer"""

    language = None

    def update(self):
        i18n = II18n(self.context)
        if self.language:
            for attr in ('title', 'body'):
                setattr(self, attr, i18n.get_attribute(attr, self.language, request=self.request))
        else:
            for attr in ('title', 'body'):
                setattr(self, attr, i18n.query_attribute(attr, request=self.request))

    render = get_view_template()
