#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.theme.interfaces import IThemesTarget, IThemesInfo, IThemesManagerTarget, IThemesManager
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_form.interfaces.form import IWidgetForm
from pyams_skin.interfaces import IInnerPage, IPageHeader
from pyams_skin.layer import IPyAMSLayer
from pyams_thesaurus.interfaces.thesaurus import IThesaurus
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION
from pyams_zmi.interfaces.menu import IPropertiesMenu
from pyams_zmi.layer import IAdminLayer

# import packages
from pyams_content.shared.common.zmi import WfSharedContentHeaderAdapter
from pyams_form.form import AJAXEditForm
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.viewlet.menu import MenuItem
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import query_utility
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm
from pyramid.view import view_config
from z3c.form import field
from zope.interface import implementer

from pyams_content import _


@viewlet_config(name='themes.menu', context=IThemesTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, permission=VIEW_SYSTEM_PERMISSION, weight=350)
class ThemesMenu(MenuItem):
    """Themes menu"""

    label = _("Themes...")
    icon_class = 'fa-tags'
    url = '#themes.html'


@pagelet_config(name='themes.html', context=IThemesTarget, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
@template_config(template='templates/themes-info.pt', layer=IPyAMSLayer)
@implementer(IWidgetForm, IInnerPage)
class ThemesEditForm(AdminEditForm):
    """Themes edit form"""

    legend = _("Content themes")

    fields = field.Fields(IThemesInfo)

    ajax_handler = 'themes.json'

    def __init__(self, context, request):
        super(ThemesEditForm, self).__init__(context, request)
        target = get_parent(self.context, IThemesManagerTarget)
        manager = IThemesManager(target)
        self.thesaurus_name = manager.thesaurus_name
        self.extract_name = manager.extract_name

    def updateWidgets(self, prefix=None):
        super(ThemesEditForm, self).updateWidgets(prefix)
        widget = self.widgets['themes']
        widget.thesaurus_name = self.thesaurus_name
        widget.extract_name = self.extract_name

    @property
    def top_terms(self):
        thesaurus = query_utility(IThesaurus, name=self.thesaurus_name)
        if thesaurus is not None:
            return sorted(thesaurus.get_top_terms(extract=self.extract_name),
                          key=lambda x: x.label)
        else:
            return ()

    def get_subterms(self, term):
        for subterm in term.specifics:
            if self.extract_name in subterm.extracts:
                yield subterm
                for another in self.get_subterms(subterm):
                    yield another


@view_config(name='themes.json', context=IThemesTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class ThemesAJAXEditForm(AJAXEditForm, ThemesEditForm):
    """Themes edit form, JSON renderer"""


@adapter_config(context=(IThemesTarget, IAdminLayer, ThemesEditForm), provides=IPageHeader)
class ThemesHeaderAdapter(WfSharedContentHeaderAdapter):
    """Shared content themes header adapter"""

    icon_class = 'fa fa-fw fa-tags'
