#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.shared.common.interfaces import IWfSharedContent, ISharedTool
from pyams_form.interfaces.form import IInnerTabForm
from pyams_i18n.interfaces import II18n
from pyams_security.interfaces import ISecurityManager
from pyams_sequence.interfaces import ISequentialIntIds, ISequentialIdInfo
from pyams_skin.layer import IPyAMSLayer
from pyams_workflow.interfaces import IWorkflowState, IWorkflow, IWorkflowStateLabel, IWorkflowVersions

# import packages
from pyams_template.template import template_config
from pyams_utils.date import format_datetime
from pyams_utils.registry import get_utility
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import contentprovider_config
from zope.interface import Interface

from pyams_content import _


@contentprovider_config(name='content_header', context=IWfSharedContent, view=Interface, layer=IPyAMSLayer)
@template_config(template='templates/header.pt', layer=IPyAMSLayer)
class SharedContentHeaderContentProvider(object):
    """Header for shared contents"""

    back_url = '#summary.html'
    back_target = None

    icon_class = ''

    def __init__(self, context, request, view):
        super(SharedContentHeaderContentProvider, self).__init__(context, request, view)
        # sequence
        sequence = get_utility(ISequentialIntIds)
        self.oid = sequence.get_short_oid(ISequentialIdInfo(context).oid)
        # security
        security = get_utility(ISecurityManager)
        owner = next(iter(context.owner))
        self.owner = security.get_principal(owner).title
        # workflow
        translate = request.localizer.translate
        workflow = IWorkflow(context)
        versions = IWorkflowVersions(context)
        state = IWorkflowState(context)
        self.version_id = state.version_id
        state_label = request.registry.queryAdapter(workflow, IWorkflowStateLabel, name=state.state)
        if state_label is None:
            state_label = request.registry.queryAdapter(workflow, IWorkflowStateLabel)
        if state_label is None:
            self.state = translate(_("{state} by {{principal}}")).format(
                state=translate(workflow.get_state_label(state.state)))
        else:
            self.state = state_label.get_label(context, request, format=False)
        principal_class = 'text-danger' if state.state_principal != owner else 'txt-color-text'
        self.state = self.state.replace('{principal}',
                                        '<span class="{0}">{{principal}}</span>'.format(principal_class))
        state_class = 'text-danger' if state.state in workflow.update_states else None
        if state_class:
            self.state = '<span class="{state_class}">{state}</span>'.format(state_class=state_class,
                                                                             state=self.state)
        self.state = self.state.format(principal=security.get_principal(state.state_principal).title)
        self.state_date = translate(_("since {date}")).format(date=format_datetime(state.state_date, request=request))
        if state.state not in workflow.update_states and versions.has_version(workflow.update_states):
            target = sorted(versions.get_versions(workflow.update_states),
                            key=lambda x: IWorkflowState(x).version_id,
                            reverse=True)[-1]
            self.version_link = {
                'css_class': 'text-danger',
                'href': absolute_url(target, request, 'admin.html'),
                'title': translate(_("access new version"))
            }
        elif state.state not in workflow.published_states and versions.has_version(workflow.published_states):
            target = sorted(versions.get_versions(workflow.published_states),
                            key=lambda x: IWorkflowState(x).version_id,
                            reverse=True)[-1]
            self.version_link = {
                'css_class': 'txt-color-text',
                'href': absolute_url(target, request, 'admin.html'),
                'title': translate(_("access published version"))
            }
        else:
            self.version_link = None

    @property
    def title(self):
        tool = get_parent(self.context, ISharedTool)
        return II18n(tool).query_attribute('title', request=self.request)


@contentprovider_config(name='content_header', context=IWfSharedContent, view=IInnerTabForm, layer=IPyAMSLayer)
class SharedContentInnerPageHeaderContentProvider(object):
    """Inner page header content provider"""

    def update(self):
        pass

    def render(self):
        return ''
