#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.interfaces import MANAGE_TOOL_PERMISSION
from pyams_content.shared.common.interfaces import ISharedTool
from pyams_form.interfaces.form import IWidgetForm, IFormHelp
from pyams_i18n.interfaces import II18n, II18nManager
from pyams_skin.interfaces import IInnerPage, IPageHeader, IContentTitle
from pyams_skin.interfaces.viewlet import IMenuHeader
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION
from pyams_zmi.interfaces.menu import IPropertiesMenu, ISiteManagementMenu
from pyams_zmi.layer import IAdminLayer

# import packages
from pyams_form.form import AJAXEditForm
from pyams_form.help import FormHelp
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.page import DefaultPageHeaderAdapter
from pyams_skin.viewlet.menu import MenuItem
from pyams_utils.adapter import adapter_config, ContextRequestViewAdapter, ContextRequestAdapter
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm
from pyramid.view import view_config
from z3c.form import field
from zope.interface import implementer, Interface

from pyams_content import _


#
# Shared tools common adapters
#

@adapter_config(context=(ISharedTool, IPyAMSLayer, Interface), provides=IContentTitle)
class SharedToolTitleAdapter(ContextRequestViewAdapter):
    """Shared tool title adapter"""

    @property
    def title(self):
        return II18n(self.context).query_attribute('title', request=self.request)


@adapter_config(context=(ISharedTool, ISiteManagementMenu), provides=IMenuHeader)
class SharedToolSiteManagementMenuHeader(ContextRequestAdapter):
    """Shared tool site management menu header adapter"""

    header = _("Tool management")


#
# Shared tool properties
#

@viewlet_config(name='properties.menu', context=ISharedTool, layer=IAdminLayer,
                manager=ISiteManagementMenu, permission=VIEW_SYSTEM_PERMISSION, weight=1)
@viewletmanager_config(name='properties.menu', layer=IAdminLayer, provides=IPropertiesMenu)
@implementer(IPropertiesMenu)
class SharedToolPropertiesMenu(MenuItem):
    """Shared tool properties menu"""

    label = _("Properties")
    icon_class = 'fa-edit'
    url = '#properties.html'


@pagelet_config(name='properties.html', context=ISharedTool, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
@implementer(IWidgetForm, IInnerPage)
class SharedToolPropertiesEditForm(AdminEditForm):
    """Shared tool properties edit form"""

    legend = _("Shared tool properties")

    fields = field.Fields(ISharedTool).omit('__parent__', '__name__')
    ajax_handler = 'properties.json'
    edit_permission = MANAGE_TOOL_PERMISSION


@view_config(name='properties.json', context=ISharedTool, request_type=IPyAMSLayer,
             permission=MANAGE_TOOL_PERMISSION, renderer='json', xhr=True)
class SharedToolPropertiesAJAXEditForm(AJAXEditForm, SharedToolPropertiesEditForm):
    """Shared tool properties edit form, JSON renderer"""


@adapter_config(context=(ISharedTool, IPyAMSLayer, SharedToolPropertiesEditForm), provides=IFormHelp)
class SharedToolPropertiesHelpAdapter(FormHelp):
    """Shared tool properties help adapter"""

    permission = MANAGE_TOOL_PERMISSION

    header = _("WARNING")
    status = 'danger'
    message = _("""Workflow shouldn't be modified if this tool already contains any shared content!""")
    message_format = 'rest'


@adapter_config(context=(ISharedTool, IPyAMSLayer, Interface), provides=IPageHeader)
class SharedToolPropertiesHeaderAdapter(DefaultPageHeaderAdapter):
    """Shared tool properties header adapter"""

    back_url = '/admin.html#properties.html'
    back_target = None

    icon_class = 'fa fa-fw fa-edit'


#
# Shared tool languages
#

@pagelet_config(name='languages.html', context=ISharedTool, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
@implementer(IInnerPage, IWidgetForm)
class SharedToolLanguagesEditForm(AdminEditForm):
    """Shared tool languages edit form"""

    legend = _("Content languages")

    fields = field.Fields(II18nManager)
    ajax_handler = 'languages.json'
    edit_permission = MANAGE_TOOL_PERMISSION


@view_config(name='languages.json', context=ISharedTool, request_type=IPyAMSLayer,
             permission=MANAGE_TOOL_PERMISSION, renderer='json', xhr=True)
class SharedToolLanguagesAJAXEditForm(AJAXEditForm, SharedToolLanguagesEditForm):
    """Shared tool languages edit form, JSON renderer"""


@adapter_config(context=(ISharedTool, IPyAMSLayer, SharedToolLanguagesEditForm), provides=IFormHelp)
class SharedToolLanguagesEditFormHelp(FormHelp):
    """Shared tool languages edit form help"""

    message = _("Tool languages are used to translate own tool properties, and newly created contents will propose "
                "these languages by default")
    message_format = 'rest'
